/*
 * These macros are used to navigate within nested structures.
 * 
 * offsetof(TYPE, MEMBER) returns the offset of the member in the type.
 * It is already in the base FreeBSD so we don't redefine it
 * 
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:        the pointer to the member.
 * @type:       the type of the container struct this is embedded in.
 * @member:     the name of the member within the struct.
 * 
 * Note that this implementation is slightly different from the linux one,
 * which triggered sme compiler warnings.
 */
#define container_of(ptr, type, member) ({			\
	__typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)( (char *)__mptr - offsetof(type,member) );})
  

